pro read_amsu_anomaly
;=====================================================================
;@***h*  MW_Sounder_FCDR/read_amsu_anomaly.pro
;
; NAME
;        read_amsu_anomaly.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              This program provides a template to be used as a guide
;              for reading in data from netCDF dataset generated by 
;              the calibration algorithm.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              netCDF files are written with the following directory 
;              structure and file format:
;
;                $InPath/AMSU_{proc}_{chan}_V1.0_YYYYMM.nc
;
; OUTPUTS:     
;              NONE.... this is a template.
;
; AUTHOR:      
;              Xinjia Zhou / Ben Ho (?)
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;  ProcLab = String in netCDF filename indicating the processing 
;            of data.
;                    values = {'COMBINED','CALIBRATED'}
;  ChanLab = 'CH09'
;  month   = The Month to read
;  year    = The Year to read
;  InPath  = The root path to the netCDF files.
;
;                       Path values should contain a trailing '/'
;
;**************************************************************************
  month=11
   year=2006
ProcLab='CALIBRATED_ANOMALY'
ChanLab='CH09'
 InPath='../Test-Data/Output/'
;*********************************************************
; End of RUN parameters
;*********************************************************

; Construct the input netCDF file name
;---------------------------------------
infile=InPath+'AMSU_'+ProcLab+'_'+ChanLab+'_V1.0_'  $
                     +string( year,format='(i4.4)') $
                     +string(month,format='(i2.2)')+'.nc'
print,infile

; Open the netCDF file, get the lon/lat dimensions
;-------------------------------------------------
ncid=ncdf_open(infile,/NOWRITE)
lonid=ncdf_dimid(ncid,'lon')
latid=ncdf_dimid(ncid,'lat')
ncdf_diminq,ncid,lonid,lon_name,lon_num
ncdf_diminq,ncid,latid,lat_name,lat_num

; Read in Variables
;-------------------
ncdf_varget,ncid,'month'      ,file_month
ncdf_varget,ncid,'year'       ,file_year
ncdf_varget,ncid,'lon'        ,lon
ncdf_varget,ncid,'lat'        ,lat
ncdf_varget,ncid,'temperature',tbs

; Close netCDF file
;------------------
ncdf_close,ncid

; DO SOME STUFF WITH THIS DATA!!
;---------------------------------
help
print,'tbs[*,36]=',tbs[*,36]
print,'tbs[36,*]=',tbs[36,*]


; End Routine
;--------------
return
end
