pro readstd,StdFile,Ps,Tps
;=====================================================================
;@***h*  MW_Sounder_FCDR/readstd.pro
;
; NAME
;        readstd.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              IDL routine to open the given file containing Standard 
;              Atmosphere data, read in the contents, interpolate values
;              to the given pressure levels, and return the results.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:   
;              StdFile = Filename containing Std. Atmosphere data
;                Ps = Pressure levels for interpolation
;
; OUTPUTS:      
;              Tps = Interpolated values
;
; AUTHOR:      
;              Xinjia Zhou / Ben Ho (?)
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================

; set dimension values
;------------------------
M0=6L
N0=40L
Ns=n_elements(Ps)

; Allocate space
;-------------------
s=''
header=strarr(444)
 T0=fltarr(N0)
 P0=fltarr(N0)
Tps=fltarr(M0,Ns)

; Open the given file
;------------------------
openr,unit,StdFile,ERROR=err,/get_lun
if(err ne 0) then begin
  print,' ERROR opening Standard Atmosphere file '
  print,' StdFile=',StdFile
  STOP
endif

; Read in data
;----------------
readf,unit,header
for ii=0,(M0-1) do begin

  ; read in 'T0' and pressure values
  ;----------------------------------
  readf,unit,s
  for jj =0,(N0-1) do begin
    readf,unit,s 
    T0[jj]=float(strmid(s,0,16))
    P0[jj]=float(strmid(s,18))
  endfor ; jj =0,(N0-1)
  readf,unit,s 

  ; Interpolate 'T0' to 'Ps' pressures
  ;------------------------------------
  Tps[ii,*]=interpol(T0,P0,Ps)
endfor ;  kk=0,(M0-1)
close,unit
free_lun,unit

; End Routine
;--------------
return
end
