      module AMSUfast
!@***h*  MW_Sounder_FCDR/AMSUfast_mod.f
!
! COPYRIGHT
!      © University Corporation for Atmospheric Research (UCAR) 2011.
!      All rights reserved. 
!      Please refer to the UCAR copyright statement in the main directory
!
!@*****
!=====================================================================



      !--------------------------------------------------------------
      ! Set all module data and routines to private by default, then 
      ! explicilty set the exposure of each subroutine/function
      !--------------------------------------------------------------
       private
       public::aqua_amsu_tran
       public::gphite
       public::calpmw
       public::conpmw
       public::taucal
       public::intrte_mws
       public::scan2zen
      

      !================
      ! Module data
      !================
       integer,parameter::NLEV=101
       integer,parameter::NLAY=NLEV-1
       integer,parameter::NKAN=19
       integer,parameter::NXD= 8
       integer,parameter::NXW=11
       integer,parameter::NCD=NXD+1
       integer,parameter::NCW=NXW+1
      
       ! Reference Atmosphere is 1976 U.S. Standard
       !----------------------------------------------
       real*4  pstd(NLEV)
       real*4  tstd(NLEV)
       real*4  wstd(NLEV)
       real*4  pavg(NLAY)
       real*4  tref(NLAY)
       real*4  wref(NLAY)

       ! Store the Current Atmospheric profile and 
       ! corresponding layer averages.
       !---------------------------------------------
       real*4  tavg(NLAY)
       real*4  wamt(NLAY)

       ! scan angle values
       !--------------------
       real*4  secz
       real*4  secp(NLAY)

       ! Coeffcients, and Wet/Dry predictors
       !--------------------------------------
       real*4  coefd(NCD,NLAY,NKAN)
       real*4  coefw(NCW,NLAY,NKAN)
       real*4  xdry(NXD,NLAY)
       real*4  xwet(NXW,NLAY)

       ! Wet/Dry transmittance profiles
       !---------------------------------
       real*4 taud(NLEV)  ! DRY (O2)  tranmittance prfile
       real*4 tauw(NLEV)  ! WET (H2O) tranmittance prfile

       ! Coefficient Filenames
       !   CoefPath = Relative path to wet/dry coefficient files
       !---------------------------------------------------------
       character*100 CoefPath
       integer       CoefPlen
       character*12  dfile
       character*12  wfile
       data          CoefPath /'./coeffl/'   /
       data          dfile    /'amsuceos.dry'/
       data          wfile    /'amsuceos.wet'/

       ! set reference atmosphere values
       !---------------------------------
       data pstd   / 0.0050,    0.0161,    0.0384,    0.0769,    0.1370,
     &    0.2244,    0.3454,    0.5064,    0.7140,    0.9753,    1.2972,
     &    1.6872,    2.1526,    2.7009,    3.3398,    4.0770,    4.9204,
     &    5.8776,    6.9567,    8.1655,    9.5119,   11.0038,   12.6492,
     &   14.4559,   16.4318,   18.5847,   20.9224,   23.4526,   26.1829,
     &   29.1210,   32.2744,   35.6505,   39.2566,   43.1001,   47.1882,
     &   51.5278,   56.1260,   60.9895,   66.1253,   71.5398,   77.2396,
     &   83.2310,   89.5204,   96.1138,  103.0172,  110.2366,  117.7775,
     &  125.6456,  133.8462,  142.3848,  151.2664,  160.4959,  170.0784,
     &  180.0183,  190.3203,  200.9887,  212.0277,  223.4415,  235.2338,
     &  247.4085,  259.9691,  272.9191,  286.2617,  300.0000,  314.1369,
     &  328.6753,  343.6176,  358.9665,  374.7241,  390.8926,  407.4738,
     &  424.4698,  441.8819,  459.7118,  477.9607,  496.6298,  515.7200,
     &  535.2322,  555.1669,  575.5248,  596.3062,  617.5112,  639.1398,
     &  661.1920,  683.6673,  706.5654,  729.8857,  753.6275,  777.7897,
     &  802.3714,  827.3713,  852.7880,  878.6201,  904.8659,  931.5236,
     &  958.5911,  986.0666, 1013.9476, 1042.2319, 1070.9170, 1100.0000/

       data tstd              / 190.19, 203.65, 215.30, 226.87, 237.83, 
     &  247.50, 256.03, 263.48, 267.09, 270.37, 266.42, 261.56, 256.40, 
     &  251.69, 247.32, 243.27, 239.56, 236.07, 232.76, 230.67, 228.71, 
     &  227.35, 226.29, 225.28, 224.41, 223.61, 222.85, 222.12, 221.42, 
     &  220.73, 220.07, 219.44, 218.82, 218.23, 217.65, 217.18, 216.91, 
     &  216.70, 216.70, 216.70, 216.70, 216.70, 216.70, 216.70, 216.70, 
     &  216.70, 216.70, 216.70, 216.70, 216.70, 216.70, 216.70, 216.71, 
     &  216.71, 216.72, 216.81, 217.80, 218.77, 219.72, 220.66, 222.51, 
     &  224.57, 226.59, 228.58, 230.61, 232.61, 234.57, 236.53, 238.48, 
     &  240.40, 242.31, 244.21, 246.09, 247.94, 249.78, 251.62, 253.45, 
     &  255.26, 257.04, 258.80, 260.55, 262.28, 264.02, 265.73, 267.42, 
     &  269.09, 270.77, 272.43, 274.06, 275.70, 277.32, 278.92, 280.51, 
     &  282.08, 283.64, 285.20, 286.74, 288.25, 289.75, 291.22, 292.68/ 

       data wstd              /  0.001,  0.001,  0.002,  0.003,  0.003, 
     &   0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003, 
     &   0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003, 
     &   0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003, 
     &   0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003, 
     &   0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003, 
     &   0.003,  0.003,  0.004,  0.004,  0.005,  0.005,  0.007,  0.009, 
     &   0.011,  0.012,  0.014,  0.020,  0.025,  0.030,  0.035,  0.047, 
     &   0.061,  0.075,  0.089,  0.126,  0.162,  0.197,  0.235,  0.273, 
     &   0.310,  0.356,  0.410,  0.471,  0.535,  0.601,  0.684,  0.784, 
     &   0.886,  0.987,  1.094,  1.225,  1.353,  1.519,  1.686,  1.852, 
     &   2.036,  2.267,  2.496,  2.721,  2.947,  3.170,  3.391,  3.621, 
     &   3.848,  4.084,  4.333,  4.579,  4.822,  5.061,  5.296,  5.528/ 

      !================
      contains
      !================

      !-----------------------------------------------------------------
       subroutine aqua_amsu_tran(temp,wvmr,theta,kanal,taut)
       !@***s*  MW_Sounder_FCDR/aqua_amsu_tran
       !
       ! PURPOSE
       ! * AQUA AMSU+HSB 101-level PLOD-based fast transmittance model
       ! .... version of 28.10.04
       !
       ! * Fast (regression) model based on line-by-line transmittances
       !   calculated with Liebe''s model.
       ! * Input temperatures and water-vapor mixing ratios must be 
       !   defined at the pressure levels in array 'pstd' 
       !   (see 'block data stdatmos').
       !
       ! * Logical units 51 & 52 are used for coefficient files.
       !
       ! INPUT:
       !	 temp = temperature profile (deg K)
       !	 wvmr = water-vapor profile (g/kg)
       !	theta = local (spacecraft) zenith angle (deg)
       !	kanal = channel number:  1-15 for AMSU-A
       !	                        16-19 for HSB
       ! OUTPUT:
       !	 taut = total transmittance profile
       !    in common/taucom/
       !	 taud = dry (O2)  transmittance profile
       !	 tauw = wet (H2O) transmittance profile
       !@*****
       !=================================================================
       integer,parameter::LENCD=NCD*NLAY
       integer,parameter::LENCW=NCW*NLAY
       integer,parameter::IOdry=51
       integer,parameter::IOwet=52
       !
       ! Passed values
       !---------------
       real*4  temp(*)
       real*4  wvmr(*)
       real*4  theta
       integer kanal
       real*4  taut(*)
       !
       ! Local Values
       !--------------
       logical newatm
       logical newang
       integer ii,jj,krec
       real*4  tlas(NLEV)
       real*4  wlas(NLEV)

       integer init
       real*4  thetl
       data thetl/-999./
       data tlas /NLEV*0./
       data wlas /NLEV*0./
       data init/1/
       save init

       ! Statement Function
       !------------------------
       secant(x)=1./cos(0.01745329*x)

       ! First time through, Read in Wet/Dry Coefs and 
       ! initialize reference values
       !----------------------------------------------------
       if(init.ne.0) then
        ! Read in Wet/Dry coefs
        !---------------------------
        CoefPlen=len_trim(CoefPath)
        open(IOdry,file=CoefPath(1:CoefPlen)//dfile,recl=LENCD*4,
     &       access='direct',status='old',err=200)
        open(IOwet,file=CoefPath(1:CoefPlen)//wfile,recl=LENCW*4,
     &       access='direct',status='old',err=200)
        do krec=1,NKAN
         read(IOdry,rec=krec) ((coefd(ii,jj,krec),ii=1,NCD),jj=1,NLAY)
         read(IOwet,rec=krec) ((coefw(ii,jj,krec),ii=1,NCW),jj=1,NLAY)
        end do
        close(IOdry)
        close(IOwet)

        ! Calculate Layer values for U.S. Reference Atmosphere
        !------------------------------------------------------
        call conpmw(pstd,tstd,wstd,NLEV,1,pavg,tref,wref)

        init=0
       endif

       ! Determine if the atmosphere profiles have changed. 
       ! If so, recalculate Layer values
       !-----------------------------------------------------
       dt=0.
       dw=0.
       do jj=1,NLEV
        if((jj.gt.1).and.(temp(jj).eq.0.)) temp(jj)=temp(jj-1)+1.0
        if((jj.gt.1).and.(wvmr(jj).eq.0.)) wvmr(jj)=wvmr(jj-1)+0.2
        dt=dt+abs(temp(jj)-tlas(jj))
        tlas(jj)=temp(jj)
        dw=dw+abs(wvmr(jj)-wlas(jj))
        wlas(jj)=wvmr(jj)
        taud(jj)=1.0
        tauw(jj)=1.0
        taut(jj)=1.0
       end do
 
       call conpmw(pstd,temp,wvmr,NLEV,1,pavg,tavg,wamt)

       ! If the scanangle value 'theta' has changed, then 
       ! recalculate dependent values
       !-----------------------------------------------------
       if(theta.eq.0.) then
        secz=1.
       else
        secz=secant(theta)
       endif
       do j=1,NLAY
        secp(j)=secz
       end do
       thetl=theta

       ! If the atmospheric profile or scan angle have changed, 
       ! then recalcualte xdry,xwet
       !--------------------------------------------------------
       call calpmw(tref,wref,tavg,wamt,pavg,secp,
     &                    NLAY,NXD,NXW,xdry,xwet)

       ! Calculate Dry, Wet, and Total Tranmittance
       !---------------------------------------------
       call taucal(NCD,NXD,NLAY,coefd(1,1,kanal),xdry,taud)
       call taucal(NCW,NXW,NLAY,coefw(1,1,kanal),xwet,tauw)
       do jj=1,(NLEV-1)
        taut(jj)=taud(jj)*tauw(jj)
       end do

       ! End Routine
       !----------------
       return
  200  print*,'ERROR: aqua_amsu_tran() opening coeffcient files'
       stop
       end subroutine aqua_amsu_tran ! (temp,wvmr,theta,kanal,taut)
      !-----------------------------------------------------------------


      !----------------------------------------------------------
       subroutine gphite(p,t,w,z_sfc,n_levels,i_dir,z)
       !@***s*  MW_Sounder_FCDR/gphite
       ! .... version of 18.05.00
       !
       ! PURPOSE:
       !  Routine to compute geopotential height given the atmospheric 
       !    state. Includes virtual temperature adjustment.
       ! CREATED:
       !  19-Sep-1996 Received from Hal Woolf, recoded by Paul van Delst
       !  18-May-2000 Logic error related to z_sfc corrected by Hal Woolf
       !  ARGUMENTS:
       !     Input
       !    --------
       !       p     - REAL*4 pressure array (mb)
       !       t     - REAL*4 temperature profile array (K)
       !       w     - REAL*4 water vapour profile array (g/kg)
       !     z_sfc   - REAL*4 surface height (m).  0.0 if not known.
       !    n_levels - INT*4 number of elements used in passed arrays
       !     i_dir   - INT*4 direction of increasing layer number
       !           i_dir = +1, Level(1) == p(top)         } satellite/AC
       !                       Level(n_levels) == p(sfc)  }    case
       !           i_dir = -1, Level(1) == p(sfc)         } ground-based
       !                       Level(n_levels) == p(top)  }    case
       !     Output
       !    --------
       !       z     - REAL*4 pressure level height array (m)
       ! COMMENTS:
       !   Dimension of height array may not not be the same as that 
       !     of the input profile data.
       !@*****
       !=======================================================================
       implicit none
       real*4,parameter::rog=29.2898
       real*4,parameter::fac=0.5*rog
       !
       ! Passed Values
       !-------------------
       integer n_levels,i_dir
       real*4  p(*),t(*),w(*),z(*)
       real*4  z_sfc
       !
       ! Local Values
       !----------------
       integer i_start,i_end,l
       real*4  v_lower,v_upper,algp_lower,algp_upper,hgt
        
       ! - Calculate virtual temperature adjustment and exponential
       !   pressure height for level above surface.  Also set
       !   integration loop bounds                            
       !------------------------------------------------------------
       if( i_dir .gt. 0 )then
        ! Data stored top down
        !----------------------
        v_lower=t(n_levels)*(1.0+(0.00061*w(n_levels)))
        algp_lower=alog(p(n_levels))
        i_start=n_levels-1
        i_end  =1
       else
        ! Data stored bottom up
        !------------------------
        v_lower=t(1)*(1.0+(0.00061*w(1)))
        algp_lower=alog(p(1))
        i_start=2
        i_end  =n_levels
       endif
        
       ! Assign surface height 
       !--------------------------
       hgt=z_sfc

       ! Following added 18 May 2000 ... 
       !  previously, z(n_levels) for downward (usual) case 
       !  was not defined
       !-----------------------------------------------------
       if(i_dir.gt.0) then
        z(n_levels)=z_sfc
       else
        z(1)=z_sfc
       endif

       ! Loop over layers always from sfc -> top --
       !------------------------------------------------
       do l=i_start,i_end,(-1*i_dir)
        ! Apply virtual temperature adjustment for upper level
        !------------------------------------------------------
        v_upper=t(l)
        if(p(l).ge.300.0) v_upper=v_upper*(1.0+(0.00061*w(l)))
 
        ! Calculate exponential pressure height for upper layer
        !-------------------------------------------------------
        algp_upper=alog(p(l))

        ! Calculate height
        !------------------
        hgt=hgt+(fac*(v_upper+v_lower)*(algp_lower-algp_upper))

        ! Overwrite values for next layer
        !----------------------------------
        v_lower=v_upper
        algp_lower=algp_upper

        ! Store heights in same direction as other data
        !------------------------------------------------
        z(l) = hgt
       end do

       ! End Routine
       !--------------
       return
       end subroutine gphite ! (p,t,w,z_sfc,n_levels,i_dir,z)
      !----------------------------------------------------------


      !----------------------------------------------------------
       subroutine calpmw(t_avg_ref,amt_wet_ref,t_avg,amt_wet,
     &                   p_avg,sec_theta,n_layers,n_dry_pred,
     &                   n_wet_pred,pred_dry,  pred_wet)
       !@***s*  MW_Sounder_FCDR/calpmw
       ! #### 100 layers!
       !
       !  PURPOSE:
       !    Routine to calculate the predictors for the dry 
       !    (temperature), and wet components of a fast transmittance 
       !     model for a scanning satellite based microwave instrument.
       !  REFERENCES:
       !    AIRS FTC package science notes and software, S. Hannon and 
       !      L. Strow, Uni. of Maryland, Baltimore County (UMBC)
       !  CREATED:
       !    19-Sep-1996 HMW
       !  ARGUMENTS:
       !      Input
       !    -----------
       !   t_avg_ref  -REAL reference layer average temperature array (K)
       !  amt_wet_ref -REAL reference water vapour amount array (k.mol)/cm^2
       !    t_avg     -REAL layer average temperature array (K)
       !   amt_wet    -REAL water vapour amount array (k.mol)/cm^2
       !    p_avg     -REAL layer average pressure array (mb)
       !  sec_theta   -REAL secant of the zenith angle array
       !   n_layers   -INT Number of atmospheric layers
       !  n_dry_pred  -INT number of dry (temperature) predictors
       !  n_wet_pred  -INT number of water vapour predictors
       !
       !      Output
       !    -----------
       !    pred_dry  -REAL dry gas (temperature) predictor matrix
       !    pred_wet  -REAL water vapour predictor matrix
       !
       !  COMMENTS:
       !    Levels or Layers?
       !    -----------------
       !      Profile data is input at a number of *LAYERS*.
       !    Layer Numbering pt. A
       !    ---------------------
       !      Layer 1   => Atmosphere between LEVELs 1 & 2
       !      Layer 2   => Atmosphere between LEVELs 2 & 3
       !      Layer L-1 => Atmosphere between LEVELs L-1 & L
       !
       !    Layer Numbering pt. B
       !    ---------------------
       !      For the HIS instrument, Layer 1 is at the top of the 
       !      atmosphere and Layer L-1 is at the surface.    
       !
       !    Layer Numbering pt. C
       !    ---------------------
       !      In this routine the number of *LAYERS* is passed in the 
       !        argument list, _not_ the number of LEVELS.  This was 
       !        done to improve the readability of this code, 
       !      i.e. loop from 1->L(ayers) rather than from 1->L(evels)-1.
       !@*****
       !=================================================================
       implicit none
       integer,parameter::max_layers=100
       integer,parameter::max_dry_pred=8
       integer,parameter::max_wet_pred=11
       !
       ! Passed Values
       !-----------------
        ! Input
        !-----------
       integer n_layers 
       integer n_dry_pred, n_wet_pred
       real*4  t_avg_ref(*), amt_wet_ref(*)
       real*4  t_avg(*),     amt_wet(*)
       real*4  p_avg(*),     sec_theta(*)

        ! Output
        !------------
       real*4  pred_dry(n_dry_pred, *)
       real*4  pred_wet(n_wet_pred, *)
       !
       ! Local variables
       !-----------------
       integer l

       real*4  p_dp(max_layers)            ! ....Pressure
       real*4  p_norm(max_layers)
 
       real*4  delta_t(max_layers)         ! ....Temperature
       real*4  t_ratio(max_layers) 
       real*4  pw_t_ratio(max_layers)      ! Pressure weighted
 
       real*4  wet_ratio(max_layers)       ! ....Water vapour
       real*4  pw_wet(max_layers)          ! Pressure weighted
       real*4  pw_wet_ref(max_layers)      ! Pressure weighted
       real*4  pw_wet_ratio(max_layers)    ! Pressure weighted

       !  -- Check that n_layers is o.k. --
       !-------------------------------------------
       if(n_layers.gt.max_layers )then
        write(*,'(/10x,''*** calpmw : n_layers > max_layers'')')
        stop
       endif 

       !  -- Check that numbers of predictors is consistent --
       !---------------------------------------------------------
        !  # of dry (temperature) predictors
        !---------------------------------
        if(n_dry_pred.ne.max_dry_pred) then
         write(*,'(/10x,''*** calpmw : invalid n_dry_pred'')')
         stop
        endif 
    
        ! # of water vapour predictors
        !----------------------------
        if(n_wet_pred.ne.max_wet_pred) then
         write(*,'(/10x,''*** calpmw : invalid n_wet_pred'')')
         stop
        endif 

       !  -- Calculate ratios, offsets, etc, for top layer --
       !--------------------------------------------------------
        ! Pressure variables
        !---------------------
        p_dp(1)  =p_avg(1)*(p_avg(2)-p_avg(1))
        p_norm(1)=0.0

        ! Temperature variables
        !---------------------
        delta_t(1)=t_avg(1)-t_avg_ref(1)
        t_ratio(1)=t_avg(1)/t_avg_ref(1)
        pw_t_ratio(1)=0.0

        ! Water vapour
        !----------------
        wet_ratio(1)   =amt_wet(1)/amt_wet_ref(1)
        pw_wet(1)      =p_dp(1)*amt_wet(1)
        pw_wet_ref(1)  =p_dp(1)*amt_wet_ref(1)
        pw_wet_ratio(1)=wet_ratio(1)

       ! Calculate ratios, offsets, etc, for all layers 
       !------------------------------------------------------
       do l = 2, n_layers

        ! Pressure variables
        !------------------
        p_dp(l)  =p_avg(l)*(p_avg(l)-p_avg(l-1))
        p_norm(l)=p_norm(l-1)+p_dp(l)

        ! Temperature variables
        !---------------------
        delta_t(l)   =t_avg(l)-t_avg_ref(l)
        t_ratio(l)   =t_avg(l)/t_avg_ref(l)
        pw_t_ratio(l)=pw_t_ratio(l-1)+(p_dp(l)*t_ratio(l-1))

        ! Water vapour
        !------------------
        wet_ratio(l) =amt_wet(l)/amt_wet_ref(l)
        pw_wet(l)    =pw_wet(l-1)+(p_dp(l)*amt_wet(l))
        pw_wet_ref(l)=pw_wet_ref(l-1)+(p_dp(l)*amt_wet_ref(l))

       end do

       ! Scale the pressure dependent variables 
       !-----------------------------------------
       do l = 2, n_layers
        pw_t_ratio(l)  =pw_t_ratio(l)/p_norm(l)
        pw_wet_ratio(l)=pw_wet(l)/pw_wet_ref(l)
       end do

       !  Load up predictor arrays 
       !----------------------------
       do l = 1, n_layers

        ! Temperature predictors
        !-------------------------
        pred_dry(1,l)=sec_theta(l)
        pred_dry(2,l)=sec_theta(l)*sec_theta(l)
        pred_dry(3,l)=sec_theta(l)*t_ratio(l)
        pred_dry(4,l)=pred_dry(3,l)*t_ratio(l)
        pred_dry(5,l)=t_ratio(l)
        pred_dry(6,l)=t_ratio(l)*t_ratio(l)
        pred_dry(7,l)=sec_theta(l)*pw_t_ratio(l)
        pred_dry(8,l)=pred_dry(7,l)/t_ratio(l) 

        ! Water vapour predictors
        !---------------------------
        pred_wet( 1,l)=sec_theta(l)*wet_ratio(l)
        pred_wet( 2,l)=sqrt(pred_wet(1,l))
        pred_wet( 3,l)=pred_wet(1,l)*delta_t(l)
        pred_wet( 4,l)=pred_wet(1,l)*pred_wet(1,l)
        pred_wet( 5,l)=abs(delta_t(l))*delta_t(l)*pred_wet(1,l)
        pred_wet( 6,l)=pred_wet(1,l)*pred_wet(4,l)
        pred_wet( 7,l)=sec_theta(l)*pw_wet_ratio(l)
        pred_wet( 8,l)=pred_wet(2,l)*delta_t(l)
        pred_wet( 9,l)=sqrt(pred_wet(2,l))
        pred_wet(10,l)=pred_wet(7,l)*pred_wet(7,l)
        pred_wet(11,l)=sqrt(pred_wet(7,l))

       end do

       ! End Routine
       !---------------
       return
       end subroutine calpmw
      !----------------------------------------------------------


      !----------------------------------------------------------
       subroutine conpmw(p,t,w,n_levels,i_dir,p_avg,t_avg,w_amt)
       !@***s*  MW_Sounder_FCDR/conpmw
       ! #### 101 levels!
       !
       !  PURPOSE:
       !    Function to convert atmospheric water vapour (g/kg) profile
       !      specified at n_levels layer BOUNDARIES to n_levels-1
       !      integrated layer amounts of units (k.moles)/cm^2 for use in
       !	 microwave fast transmittance models. The average LAYER
       !      pressure and temperature are also returned.
       !  REFERENCES:
       !    AIRS LAYERS package science notes, S. Hannon and L. Strow, 
       !      Uni. of Maryland, Baltimore County (UMBC)
       !  CREATED:
       !    19-Sep-1996 HMW
       !  ARGUMENTS:
       !     Input
       !    --------
       !       p     - REAL*4 pressure array (mb)
       !       t     - REAL*4 temperature profile array (K)
       !       w     - REAL*4 water vapour profile array (g/kg)
       !    n_levels - INT*4 number of elements used in passed arrays
       !     i_dir   - INT*4 direction of increasing layer number
       !            i_dir = +1, Level(1) == p(top)         } satellite/AC
       !                        Level(n_levels) == p(sfc)  }    case
       !            i_dir = -1, Level(1) == p(sfc)         } ground-based
       !                        Level(n_levels) == p(top)  }    case
       !     Output
       !    --------
       !     p_avg   - REAL average LAYER pressure array (mb)
       !     t_avg   - REAL average LAYER temperature (K)
       !     w_amt   - REAL integrated LAYER water vapour amount 
       !               array (k.moles)/cm^2
       !
       !  ROUTINES:
       !    Subroutines:
       !    ------------
       !      gphite  - calculates geopotential height given profile data.
       !    Functions:
       !    ----------
       !      NONE
       !  COMMENTS:
       !    Levels or Layers?
       !    -----------------
       !      Profile data is input at a number of *LEVELS*.  
       !        Number densitites are calculated for *LAYERS* that are 
       !        bounded by these levels. So, for L levels there are 
       !        L-1 layers.
       !    Layer Numbering
       !    ---------------
       !      Layer 1   => Atmosphere between LEVELs 1 & 2
       !      Layer 2   => Atmosphere between LEVELs 2 & 3
       !      Layer L-1 => Atmosphere between LEVELs L-1 & L
       !@*****
       !================================================================
       implicit none
       integer,parameter::max_levels=101        ! Maximum number of levels
       real*4,parameter::r_equator=6.378388e+06 ! Earth radius at equator
       real*4,parameter::r_polar  =6.356911e+06 ! Earth radius at pole
       real*4,parameter::r_avg=0.5*(r_equator+r_polar)
       real*4,parameter::g_sfc=9.80665          ! Gravity at surface
       real*4,parameter::rho_ref=1.2027e-12     ! Reference air "density"
       real*4,parameter::mw_dryair=28.97        ! Mol. wgt. dry air (g/mol)
       real*4,parameter::mw_h2o=18.0152         ! Mol. wgt. water
       real*4,parameter::R_gas=8.3143           ! Ideal gas const(J/mole/K)
       real*4,parameter::R_air=0.9975*R_gas     ! Gas const air(worst case) 
       !
       ! Passed Values
       !-----------------
       integer n_levels, i_dir
       real*4  p(*),t(*),w(*)
       real*4  p_avg(*),t_avg(*),w_amt(*)
       !
       ! Local Values
       !---------------
       integer l,l_start,l_end,l_indx
       real*4  rho1,rho2,p1,p2,w1,w2,z1,z2
       real*4  c_avg,g_avg,z_avg,w_avg
       real*4  dz,dp,r_hgt,wg,A,B

       real*4  z(max_levels)           ! Pressure heights (m)
       real*4  g(max_levels)           ! Acc. due to gravity (m/s/s)
       real*4  mw_air(max_levels)      ! Molec. wgt. of air (g/mol)
       real*4  rho_air(max_levels)     ! air mass density (kg.mol)/m^3
       real*4  c(max_levels)           ! (kg.mol.K)/(N.m)
       real*4  w_ppmv(max_levels)      ! h2o LEVEL amount (ppmv)

       ! Calculate initial values of pressure heights 
       !-----------------------------------------------
       call gphite( p, t, w, 0.0, n_levels, i_dir, z)

       ! Set loop bounds for direction sensitive calculations 
       ! so loop iterates from surface to the top             
       !------------------------------------------------------
       if( i_dir .gt. 0 )then
        ! Data stored top down
        !--------------------
        l_start=n_levels
        l_end  =1
       else
        ! Data stored bottom up
        !---------------------
        l_start=1
        l_end  =n_levels
       endif

       ! Air molecular mass and density, and gravity 
       ! as a function of LEVEL. Loop from bottom to top
       !-----------------------------------------------
       do l=l_start,l_end,(-1*i_dir)
        ! Convert water vapour g/kg -> ppmv
        !---------------------------------
        w_ppmv(l) = 1.0e+03 * w(l) * mw_dryair / mw_h2o

        ! Calculate molecular weight of air (g/mol)
        !----------------------------------------
        mw_air(l)=((1.0-(w_ppmv(l)/1.0e+6))*mw_dryair)
     &                +((w_ppmv(l)/1.0e+6)*mw_h2o)

        ! Air mass density
        !-------------------
        c(l)=0.001*mw_air(l)/R_air    ! 0.001 factor for g -> kg
        rho_air(l)=c(l)*p(l)/t(l)

        ! Gravity
        !----------
        r_hgt=r_avg+z(l)                                     !  m
        g(l)=g_sfc-g_sfc*(1.0-((r_avg*r_avg)/(r_hgt*r_hgt))) !  m/s^2
       end do

       ! LAYER quantities. Loop from bottom to top
       !-----------------------------------------------
       do l=l_start,l_end+i_dir,(-1*i_dir)
        ! Determine output array index.  This is done so that the
        ! output data is always ordered from 1 -> L-1 regardless
        ! of the orientation of the input data.  This is true by
        ! default only for the bottom-up case.  For the top down
        ! case no correction would give output layers from 2 -> L
        !-------------------------------------------------------
        if( i_dir .gt. 0 )then
         l_indx=l-1
        else
         l_indx=l
        endif

        ! Assign current layer boundary densities
        !------------------------------------------
        rho1 = rho_air(l)
        rho2 = rho_air(l-i_dir)
 
        ! Average c
        !------------
        c_avg=((rho1*c(l))+(rho2*c(l-i_dir)))/(rho1+rho2)

        ! Average t
        !------------
        t_avg(l_indx)=((rho1*t(l))+(rho2*t(l-i_dir)))/(rho1+rho2)

        ! Average p
        !------------
        p1=p(l)
        p2=p(l-i_dir)

        z1=z(l)
        z2=z(l-i_dir)

        dp=p2-p1

        A=log(p2/p1)/(z2-z1)
        B=p1/exp(A*z1)

        p_avg(l_indx)=dp/log(p2/p1)

        ! LAYER thickness (rather long-winded as it is not
        ! assumed the layers are thin) in m. Includes
        ! correction for altitude/gravity.
        !----------------------------------------------------
         !...Initial values
         g_avg=g(l)
         dz =-1.0*dp*t_avg(l_indx)/(g_avg*c_avg*p_avg(l_indx))
 
         !...Calculate z_avg
         z_avg=z(l)+(0.5*dz)
 
         !...Calculate new g_avg
         r_hgt=r_avg+z_avg 
         g_avg=g_sfc-g_sfc*(1.0-((r_avg*r_avg)/(r_hgt*r_hgt)))
 
         !...Calculate new dz
         dz=-1.0*dp*t_avg(l_indx)/(g_avg*c_avg*p_avg(l_indx))
 
        ! Calculate LAYER amounts for water vapour
        !-------------------------------------------
        w1=w_ppmv(l)
        w2=w_ppmv(l-i_dir)

        w_avg=((rho1*w1)+(rho2*w2))/(rho1+rho2)

        w_amt(l_indx)=rho_ref*w_avg*dz*p_avg(l_indx)/t_avg(l_indx)

       end do ! l=l_start,l_end+i_dir,(-1*i_dir)

       !  End Routine
       !------------------
       return
       end subroutine conpmw
      !----------------------------------------------------------


      !----------------------------------------------------------
       subroutine taucal(nc,nx,ny,cc,xx,tau)
       !@***s*  MW_Sounder_FCDR/taucal
       ! * General transmittance calculation ... Strow-Woolf model
       !@*****
       !===========================================================
       !
       ! Passed Values
       !-----------------
       integer nc,nx,ny
       real*4  cc(nc,ny),xx(nx,ny),tau(*)
       real*4  trap
       data trap/-999.99/
       !
       ! Local Values
       !-----------------
       integer ii,jj
       integer last
       real*4  yy
       real*4  taulyr
       real*4  taulev

       !write(*,*)'trap:',trap
       !write(*,*)nc,nx,ny
       !write(*,*)cc
       !if(nc .eq. 9)write(*,*)(xx(1,jj),jj=1,100)

       last=ny
       taulyr=1.
       taulev=1.
       tau(1)=taulev
       do jj=1,ny 
        if(taulev.eq.0.) goto 110
        if(jj.gt.last) then
         !write(*,*)'I am last'
         goto 100
        endif  
        yy=cc(nc,jj)
        !write(*,*)'yy:',yy
        if(yy.eq.trap) then
         !write(*,*)'I am trap'
         last=jj
         goto 100
        endif
        do ii=1,nx
         yy=yy+cc(ii,jj)*xx(ii,jj)
         !if(nc .eq. 9 .and. jj.le.15) then
         !    write(*,*)jj,yy,cc(ii,jj),xx(ii,jj)
         !endif
        end do
        yy=amax1(yy,0.)
        if(yy.gt.0.) taulyr=exp(-yy)
  100   taulev=taulev*taulyr
        !if(nc .eq. 9)write(*,*)jj,taulev,taulyr
  110   tau(jj+1)=taulev
       end do
       !if(nc .eq. 9) write(*,*)'tau:',(tau(ii),ii=1,10)

       ! End Routine
       !----------------
       return
       end subroutine taucal
      !----------------------------------------------------------


      !----------------------------------------------------------
       subroutine intrte_mws(tran,temp,tsfc,esfc,lsfc,
     &                            tbbd,tbbr,tbbs,tbbt)
       !@***s*  MW_Sounder_FCDR/intrte_mws
	! DESCRIPTION
       ! * Integrate RTE for a microwave sounder with surface reflection
       ! .... version of 28.10.04
       !
       ! Inputs:
       !      tran = profile of transmittance (~1.0 at top)
       !      temp = profile of temperature   (deg K)
       !      tsfc = surface temperature      (deg K)
       !      esfc = surface emissivity       (0.0 --> 1.0)
       !      lsfc = surface level number     (101 = bottom [1100mb])
       ! Outputs:
       !      tbbd = direct (upwelling) component
       !      tbbr = reflected component
       !      tbbs = surface component
       !      tbbt = total brightness temperature
       !@*****
       !================================================
       !
       !  Passed values
       !-------------------
       real*4  tran(*),temp(*)
       real*4  tsfc,esfc
       integer lsfc
       real*4  tbbd,tbbr,tbbs,tbbt
       !
       ! Local Values
       !---------------
       integer n,m,l
       real*4  taus,taum,taup,tt
       integer lm1,lp1

       ! Set surface values
       !---------------------
       n=lsfc
       m=n-1
       taus=tran(n)

       ! * accumulators for direct, reflected, and surface components
       !---------------------------------------------------------------
       tbbd=temp(1)*(tran(1)-tran(2))
       tbbr=0.0
       tbbs=0.0

       do l=2,m
        lm1=l-1
        taum=tran(lm1)
        lp1=l+1
        taup=tran(lp1)
        tt=temp(l)
        tbbd=tbbd+tt*(taum-taup)
        if(taus.eq.0.0) cycle
        if(taum.eq.0.0) cycle
        if(taup.eq.0.0) cycle
        if(esfc.eq.1.0) cycle
        tbbr=tbbr+tt*(taus/taup-taus/taum)
       end do ! l=2,m

       tt=temp(n)
       taum=tran(m)
       tbbd=tbbd+tt*(taum-taus)
       tbbd=0.5*tbbd

       ! chick if there is no reflected or surface component
       !------------------------------------------------------
       if(taus.eq.0.0) then
        tbbt=tbbd
        return
       endif

       ! check id there is no reflected component
       !--------------------------------------------
       tbbs=esfc*tsfc*taus
       if((taum.eq.0.0).or.(esfc.eq.1.0)) then
        tbbt=tbbd+tbbs
        return
       endif

       ! finalize reflected component
       !---------------------------------
       tbbr=tbbr+temp(1)*(taus/tran(2)-taus/tran(1))+tt*(1.-taus/taum)

       ! ... assume reflectivity = 1 - emissivity
       !-------------------------------------------
       tbbr=0.5*tbbr*taus*(1.0-esfc)

       ! End Routine
       !--------------
       tbbt=tbbd+tbbr+tbbs
       return
       end subroutine intrte_mws
      !----------------------------------------------------------


      !----------------------------------------------------------
       real*4 function scan2zen(scang,altkm)
       !@***f*  MW_Sounder_FCDR/scan2zen
	! FUNCTION
       ! * Convert scan (nadir) angle to local zenith angle 
       !   for the given altitude
       !   .... version of 21.05.03
       !
       ! * Inputs:
       !     altkm = altitude in kilometers
       !     scang = scan angle in degrees
       !
       ! * Output (function name):
       !           = local zenith angle in degrees
       !@*****
       !===========================================
       real*4,parameter::radius=6371.03
       real*4,parameter::dtr=.01745329
       real*4  scang,altkm

       scan2zen=(asin(sin(scang*dtr)*(radius+altkm)/radius))/dtr

       ! End Function
       !---------------
       return
       end function scan2zen
      !----------------------------------------------------------

      end module
