      program main
!=====================================================================
!@***h*  MW_Sounder_FCDR/gen_netCDF_climatology.f
!
! NAME
!           gen_netCDF_climatology.f
!
! LOCATION
!           $SRC_DIR/Src_fortran/
!
! DESCRIPTION: 
!              Read in gridded ASCII data for the specified months,
!              write out desired values to netCDF files.The original 
!              IDL program had to be rewritten in FORTRAN to produce
!              Version 4 netCDF files. The cumbersome usage methodology 
!              is the same as for the IDL routines for consistency.
!
! EXTERNALS:   
!              none.
!
! USAGE:       
!              The RUN parameters specifying the data to process are set 
!              directly below this header. The user needs to edit these 
!              values to process the desired values and then execute the 
!              program from within IDL.
!
! INPUTS:      
!              Gridded combined polar orbiter data values are read from an 
!              ASCII file:
!
!                $InPath/tls_climatology_1.0
!
! OUTPUTS:     
!              netCDF files are written with the following directory 
!              structure and file format:
!
!                $OutPath/AMSU_{proc}_{chan}_V1.0_MM.nc
!
! AUTHOR:      
!              Xinjia Zhou / Ben Ho (?)
!
! REVISIONS:   
!              Patrick Callaghan (Formatted for CDR)
!
!
! COPYRIGHT
!              © University Corporation for Atmospheric Research (UCAR) 2011.
!              All rights reserved. Please refer to the UCAR copyright 
!              statement in the main directory
!
!@*****
!=====================================================================
      include 'netcdf.inc'

      integer,parameter::NSAT=6
      character*6   I_satnam(NSAT)
      integer       I_satnum(NSAT)
      character*300 SourceNam
      integer       SourceLen

      character*40  chan
      integer       clen
      character*130 InPath,OutPath
      integer       InPlen,OutPlen
      character*40  ProcLabT,ProcLab
      integer       ProcLenT,ProcLen
      character*9   ChanLab
      integer       YEAR_BEG,MONTH_BEG
      integer       YEAR_END,MONTH_END
   
      character*130 infile
      integer       inflen
      character*20  time_beg,time_end
      character*12  outfbas
      character*130 outname
      integer       outnlen
      character*130 outfile
      integer       outflen

      real,   allocatable::time(:)
      real,   allocatable::lat(:)
      real,   allocatable::lon(:)
      real,   allocatable::climatology_bounds(:,:)
      real,   allocatable::lat_bounds(:,:)
      real,   allocatable::lon_bounds(:,:)
      real,   allocatable::tbs(:,:,:)
      integer              I_month

      real    lat_loop,lon_loop
      real    lat_res,lon_res
      integer lat_num,lon_num
      integer time_num,bnds_num
      integer lat_ind,lon_ind
      integer lon_lo,lon_hi
      integer inum,lun3,ioval
      integer month_loop
      real    time_min,time_max

      character*30 timelab
      character*20 cdate
      integer values(8)
      integer retval,Vdim(3)
      integer ncid,timedim,bndsdim,londim,latdim
      integer time_id ,lat_id, lon_id
      integer time_bid,lat_bid,lon_bid
      integer tbsid

      data I_satnum / 1, 1, 1, 1, 1, 1 /
      data I_satnam /'noaa16', 'noaa17', 'noaa18', 
     &               'noaa19', 'noaam2', '  aqua'/

      real    Jdays(12)     ,Jlo(12)     ,Jhi(12)
      real    Jdays_leap(12),Jlo_leap(12),Jhi_leap(12)
      data Jdays_leap /  15. ,  45. ,  75. , 105.5, 136. , 166.5,
     &                  197. , 228. , 258.5, 289. , 319.5, 350. /
      data Jdays      /  15. ,  44.5,  74. , 104.5, 135. , 165.5,
     &                  196. , 227. , 257.5, 288. , 318.5, 349. /
      data Jlo_leap /   0. ,  31. ,  60. ,  91. , 121. , 152. ,
     &                182. , 213. , 244. , 274. , 305. , 335. /
      data Jhi_leap /  30. ,  59. ,  90. , 120. , 151. , 181. ,
     &                212. , 243. , 273. , 304. , 334. , 365. /
      data Jlo      /   0. ,  31. ,  59. ,  90. , 120. , 151. ,
     &                181. , 212. , 243. , 273. , 304. , 334. /
      data Jhi      /  30. ,  58. ,  89. , 119. , 150. , 180. ,
     &                211. , 242. , 272. , 303. , 333. , 364. /

!**************************************************************************
! Set RUN parameters here:
!                 chan= String indicating the AMSU channel number and whether 
!                       the data has been converted using GPSRO data.
!                           values = {'chan9', 'chan9_converted'}
!  YEAR_BEG,MONTH_BEG = The begining year and Month of climatology
!  YEAR_END,MONTH_END = The ending year and Month of climatology
!   InPath            = The root path to the ASCII input climatology file.
!   OutPath           = The root path for the netCDF files containing 
!                       combined/calibrated climatology data
!---------------
!  USER BEWARE!
!---------------
!  ** {chan,YEAR_BEG,MONTH_BEG,YEAR_END,MONTH_END} should be set to be 
!  ** consistent with values used to generate the data in the climatology
!  ** dataset, these values are not contained in the dataset and there is 
!  ** no checking to verify accuaracy of these values, they just get 
!  ** written to the netCDF file.
!
!                       Path values should contain a trailing '/'
!
!**************************************************************************
           chan='chan9_converted'
       YEAR_BEG=2002
      MONTH_BEG=1 
       YEAR_END=2010
      MONTH_END=12
         InPath='../Test-Data/ASCII_tmp/Climatology/'
        OutPath='../Test-Data/Output/'
!*********************************************************
! End of RUN parameters
!*********************************************************

!
! Set RUN string lengths
!------------------------
      clen=len_trim(chan)
      InPlen=len_trim(InPath)
      OutPlen=len_trim(OutPath)
!
! Set the creation date
!--------------------------
      call date_and_time(VALUES=values)
      cdate='YYYY-MM-DDTHH:MM:SSZ'
      write(cdate( 1: 4),'(i4.4)') values(1)
      write(cdate( 6: 7),'(i2.2)') values(2)
      write(cdate( 9:10),'(i2.2)') values(3)
      write(cdate(12:13),'(i2.2)') (values(5)-(values(4)/60))
      write(cdate(15:16),'(i2.2)') values(6)
      write(cdate(18:19),'(i2.2)') values(7)
!
! Set netCDF filename strings for specified {chan}
!      ProcLab = String in netCDF filename indicating the processing 
!                of data.
!                     values = {'COMBINED','CALIBRATED'}
!      ChanLab = 'CH09'
!--------------------------------------------------
      if(chan(7:15).eq.'converted') then
        ProcLab='RO_CAL_BT_V01R00_CLIM_'
        ProcLabT='Calibrated_Climatology'
      else
        ProcLab='RO_COM_BT_V01R00_CLIM_'
        ProcLabT='Combined_Climatology'
      endif
      ProcLen =len_trim(ProcLab)
      ProcLenT=len_trim(ProcLabT)
      if(chan(1:5).eq.'chan9') then 
        ChanLab='AMSU_CH9_'
      else 
        print *,' Unknown channel'
        STOP
      endif
 
! Construct the begining/ending time strings 
! for netCDF attributes
!-----------------------------------------
      time_beg='YYYY-MM-01T00:00:00Z'
      time_end='YYYY-MM-31T23:59:59Z'
      write(time_beg(1:4),'(i4.4)') YEAR_BEG
      write(time_beg(6:7),'(i2.2)') MONTH_BEG
      write(time_end(1:4),'(i4.4)') YEAR_END
      write(time_end(6:7),'(i2.2)') MONTH_END
 
! Generate list of source satellites
!--------------------------------------
      SourceNam='UCAR COSMIC Data Analysis and Archival Center vers'
     &        //'ion 2010.2640; NOAA archive of Level 1b AMSU datas'
     &        //'et; NASA DAAC archive of Aqua Level 1b dataset;   '
     &        //'Polar Orbiters used:                              '
     &        //'                                                  '
      SourceLen=171
      do inum=1,NSAT
       if(I_satnum(inum).gt.0) then
        SourceLen=SourceLen+8
        SourceNam((SourceLen-6):SourceLen)=I_satnam(inum)
       endif
      end do
 
! Set the Lat/Lon resolution
!------------------------------
      lat_res=2.5
      lon_res=2.5

! Get lon/lat grid dimensions, allocate space,
! and initialize values tbs() values.
!-----------------------------------------------
      time_num=12
      bnds_num=2
      lat_num=ifix(180./lat_res)
      lon_num=ifix(360./lon_res)
      allocate(time(time_num))
      allocate( lat(lat_num))
      allocate( lon(lon_num))
      allocate(climatology_bounds(bnds_num,time_num))
      allocate(        lat_bounds(bnds_num,lat_num))
      allocate(        lon_bounds(bnds_num,lon_num))
      allocate(tbs(lon_num,lat_num,time_num))
      tbs(:,:,:)=-999.

! Load Lat/Lon/Time values
!--------------------------
      inum=1
      lat_loop=-90. 
      do while(lat_loop.le.(90.-lat_res))  
        lat(inum)=lat_loop+(lat_res/2.)
        lat_bounds(1,inum)=lat(inum)-(lat_res/2.)
        lat_bounds(2,inum)=lat(inum)+(lat_res/2.)
        inum=inum+1
        lat_loop=lat_loop+lat_res
      end do  ! while(lat_loop.le.(90.-lat_res))

      inum=1
      lon_loop=-180.
      do while(lon_loop.le.(180.-lon_res)) 
        lon(inum)=lon_loop+(lon_res/2.)
        lon_bounds(1,inum)=lon(inum)-(lon_res/2.)
        lon_bounds(2,inum)=lon(inum)+(lon_res/2.)
        inum=inum+1
        lon_loop=lon_loop+lon_res
      end do  ! while(lon_loop.le.(180.-lon_res))

      do inum=1,12
        time(inum)=Jdays(inum)
        climatology_bounds(1,inum)=daysince(YEAR_BEG,YEAR_BEG,Jlo(inum))
        climatology_bounds(2,inum)=daysince(YEAR_BEG,YEAR_END,Jhi(inum))
      end do
      time_min=0.
      time_max=climatology_bounds(2,12)

! Construct the input file name
!-------------------------------
      infile=InPath(1:InPlen)//'tls_climatology_1.0'
      inflen=len_trim(infile)
      print *,infile(1:inflen)

! Open the ASCII climatology file.
!--------------------------------------
      lun3=13
  901 format(16(f8.3,2x))
  902 format(i8)
      open(unit=lun3,file=infile(1:inflen),status='unknown',
     &     form='formatted',action='read')

      ! Loop over months
      !--------------------
      do month_loop=1,12

        ! Read in header record containing observation 
        ! counts for NSAT satellites.
        !---------------------------------------------------
        read(lun3,902,iostat=ioval) I_month
        if(ioval.gt.0) then
          print *,' I/O Input error reading header: err=',ioval
          stop
        elseif(ioval.lt.0) then
          print *,' EOF reached reading header: ioval=',ioval
          stop
        elseif(I_month.ne.month_loop) then
          print *,' ERROR: Month mismatch in climatology file'
          print *,'  month_loop=',month_loop,' I_month=',I_month
          stop
        endif

        ! Read in lat,lon data
        !----------------------------
        do lat_ind=1,72 
          do lon_ind=1,9 
            lon_lo=1+(16*(lon_ind-1))
            lon_hi=lon_lo+15
            read(lun3,901,iostat=ioval) 
     &                            tbs(lon_lo:lon_hi,lat_ind,month_loop)
            if(ioval.gt.0) then
              print *,' I/O Input error while reading climatology data:'
              print *,'    err=',ioval
              stop
            elseif(ioval.lt.0) then
              print *,' EOF reached while reading climatology data:'
              print *,'    ioval=',ioval
              stop
            endif
          end do
        end do
      end do  ! month_loop=1,12
      close(lun3)
!
! Write data out to netCDF file
!---------------------------------
      
      ! Construct the output netCDF file name
      !---------------------------------------
      outfbas='YYYY_YYYY.nc'
      write(outfbas( 1:4 ),'(i4.4)') YEAR_BEG
      write(outfbas( 6:9 ),'(i4.4)') YEAR_END
      outname=ChanLab//ProcLab(1:ProcLen)//outfbas
      outfile=OutPath(1:OutPlen)//outname
      outnlen=len_trim(outname)
      outflen=len_trim(outfile)
      
      ! Create netCDF file
      !-------------------------
      retval=nf_create(outfile(1:outflen),nf_clobber,ncid)
      print *,' out=',outfile(1:outflen)
      print *,' '
      if(retval.ne.nf_noerr) then
        print *,' ERROR: Opening netCDF file. retval=',retval
        stop
      endif
      
      ! Set global attributes
      !------------------------
      retval=nf_put_att_text(ncid,nf_global,
     &       'Conventions',6,'CF-1.5')
      retval=nf_put_att_text(ncid,nf_global,
     &       'title',(ProcLenT+50),
     &       ProcLabT(1:ProcLenT)//
     &       ' AMSU Channel 9 Brightness Temperature Climatology')
      retval=nf_put_att_text(ncid,nf_global,
     &       'product_version',6,'v01r00')
      retval=nf_put_att_text(ncid,nf_global,
     &       'source',SourceLen,SourceNam(1:SourceLen))
      retval=nf_put_att_text(ncid,nf_global,
     &       'Metadata_Conventions',63,
     &       'CF-1.5, Unidata Dataset Discovery v1.0, NOAA CDR '//
     &       'v1.0, GDS v2.0')
      retval=nf_put_att_text(ncid,nf_global,
     &       'standard_name_vocabulary',43,
     &       'CF Standard Name Table (v17, 24 March 2011)')
      retval=nf_put_att_text(ncid,nf_global,
     &       'id',outnlen,outname(1:outnlen))
      retval=nf_put_att_text(ncid,nf_global,
     &       'naming_authority',13,'gov.noaa.ncdc')
      retval=nf_put_att_text(ncid,nf_global,'date_created',20,cdate)
      retval=nf_put_att_text(ncid,nf_global,
     &       'license',36,'No constraints on data access or use')
      retval=nf_put_att_text(ncid,nf_global,
     &       'summary',314,
     &       'The product is the monthly mean climatology of adjust'//
     &       'ed AMSU channel 9 brightness temperatures. Calculated'//
     &       ' brightness temperatures from an AMSU forward model u'//
     &       'sing temperature and moisture profiles from GPSRO obs'//
     &       'ervations were used to calibrate the channel 9 measur'//
     &       'ements from polar orbiters to a common reference.')
      retval=nf_put_att_text(ncid,nf_global,
     &       'keywords',57,
     &       'Spectral/Engineering > Microwave > Brightness '//
     &       'Temperature')
      retval=nf_put_att_text(ncid,nf_global,
     &       'keywords_vocabulary',78,
     &       'NASA Global Change Master Directory (GCMD) Earth Sc'//
     &       'ience Keywords, Version 6.0')
      retval=nf_put_att_text(ncid,nf_global,
     &       'cdm_data_type',4,'Grid')
      retval=nf_put_att_text(ncid,nf_global,
     &       'institution',11,'UCAR/COSMIC')
      retval=nf_put_att_text(ncid,nf_global,
     &       'geospatial_lat_min',6,'-88.75')
      retval=nf_put_att_text(ncid,nf_global,
     &       'geospatial_lat_max',6,'+88.75')
      retval=nf_put_att_text(ncid,nf_global,
     &       'geospatial_lon_min',7,'-178.75')
      retval=nf_put_att_text(ncid,nf_global,
     &       'geospatial_lon_max',7,'+178.75')
      retval=nf_put_att_text(ncid,nf_global,
     &       'geospatial_lat_units',13,'degrees_north')
      retval=nf_put_att_text(ncid,nf_global,
     &       'geospatial_lon_units',12,'degrees_east')
      retval=nf_put_att_text(ncid,nf_global,
     &       'geospatial_lat_resolution',3,'2.5')
      retval=nf_put_att_text(ncid,nf_global,
     &       'geospatial_lon_resolution',3,'2.5')
      retval=nf_put_att_text(ncid,nf_global,
     &       'time_coverage_start',20,time_beg)
      retval=nf_put_att_text(ncid,nf_global,
     &       'time_coverage_end',20,time_end)
      retval=nf_put_att_text(ncid,nf_global,
     &       'time_coverage_duration',3,'P1Y')
      retval=nf_put_att_text(ncid,nf_global,
     &       'time_coverage_resolution',3,'P1M')
      retval=nf_put_att_text(ncid,nf_global,
     &       'cdr_program',57,
     &       'NOAA Climate Data Record Program for satellites,'//
     &       ' FY 2011.')
      retval=nf_put_att_text(ncid,nf_global,
     &       'cdr_variable',14,'bt_climatology')
      retval=nf_put_att_text(ncid,nf_global,
     &       'metadata_link',20,'gov.noaa.ncdc:C00807')
      retval=nf_put_att_text(ncid,nf_global,
     &        'platform',490,
     &        'EOS AQUA > Earth Observing System AQUA; METOP > Mete'//
     &        'orological Operational Satellite; NOAA-15 > National'//
     &        ' Oceanic & Atmospheric Administration-15; NOAA-16 > '//
     &        'National Oceanic & Atmospheric Administration-16; NO'//
     &        'AA-18 > National Oceanic & Atmospheric Administratio'//
     &        'n-18; NOAA-19 > National Oceanic & Atmospheric Admin'//
     &        'istration-19; CHAMP > Challenging Minisatellite Payl'//
     &        'oad, Gravity Recovery and Climate Experiment; COSMIC'//
     &        ' > Constellation Observing System for Meteorology, I'//
     &        'onosphere, and Climate')
      retval=nf_put_att_text(ncid,nf_global,
     &        'sensor',75,
     &        'AMSU-A > Advanced Micowave Sounding Unit-A; GPS > Gl'//
     &        'obal Positioning System')
      retval=nf_put_att_text(ncid,nf_global,
     &       'spatial_resolution',10,'2.5 degree')
      retval=nf_put_att_text(ncid,nf_global,
     &       'processing_level',12,'NASA Level 3')
     
      ! define dimensions
      !-------------------
      retval=nf_def_dim(ncid,'time',time_num,timedim)
      retval=nf_def_dim(ncid,'lon' , lon_num, londim)
      retval=nf_def_dim(ncid,'lat' , lat_num, latdim)
      retval=nf_def_dim(ncid,'bnds',bnds_num,bndsdim)

      ! Time Coordinates
      !-------------------------
      timelab='days since YYYY-01-01 00:00:00'
      write(timelab(12:15),'(i4.4)') YEAR_BEG
      retval=nf_def_var(ncid,'time',nf_float,1,timedim,time_id)
      retval=nf_put_att_text(ncid,time_id,'units',30,timelab)
      retval=nf_put_att_text(ncid,time_id,'long_name',4,'time')
      retval=nf_put_att_text(ncid,time_id,'standard_name',4,'time')
      retval=nf_put_att_text(ncid,time_id,'axis',1,'T')
      retval=nf_put_att_text(ncid,time_id,'climatology',18,
     &                                    'climatology_bounds')
      retval=nf_put_att_real(ncid,time_id,
     &                    'missing_value',nf_float,1,-999.)
     
      ! Longitude Coordinates
      !-------------------------
      retval=nf_def_var(ncid,'lon',nf_float,1,londim,lon_id)
      retval=nf_put_att_text(ncid,lon_id,'units',12,'degrees_east')
      retval=nf_put_att_text(ncid,lon_id,
     &                       'long_name',9,'longitude')
      retval=nf_put_att_text(ncid,lon_id,
     &                   'standard_name',9,'longitude')
      retval=nf_put_att_text(ncid,lon_id,'axis',1,'X')
      retval=nf_put_att_text(ncid,lon_id,'bounds',10,'lon_bounds')
      retval=nf_put_att_real(ncid,lon_id,
     &                   'missing_value',nf_float,1,-999.)
     
      ! Latitude Coordinates
      !-------------------------
      retval=nf_def_var(ncid,'lat',nf_float,1,latdim,lat_id)
      retval=nf_put_att_text(ncid,lat_id,'units',13,'degrees_north')
      retval=nf_put_att_text(ncid,lat_id,
     &                       'long_name',8,'latitude')
      retval=nf_put_att_text(ncid,lat_id,
     &                   'standard_name',8,'latitude')
      retval=nf_put_att_text(ncid,lat_id,'axis',1,'Y')
      retval=nf_put_att_text(ncid,lat_id,'bounds',10,'lat_bounds')
      retval=nf_put_att_real(ncid,lat_id,
     &                   'missing_value',nf_float,1,-999.)

      ! Time Bounds
      !-------------------------
      Vdim(1)=bndsdim
      Vdim(2)=timedim
      retval=nf_def_var(ncid,'climatology_bounds',nf_float,
     &                                              2,Vdim,time_bid)
      retval=nf_put_att_real(ncid,time_bid,
     &                         'valid_min',nf_float,1,time_min)
      retval=nf_put_att_real(ncid,time_bid,
     &                         'valid_max',nf_float,1,time_max)
      retval=nf_put_att_text(ncid,time_bid,'coordinates',4,'time')

      ! Longitude Bounds
      !-------------------------
      Vdim(1)=bndsdim
      Vdim(2)=londim
      retval=nf_def_var(ncid,'lon_bounds',nf_float,2,Vdim,lon_bid)
      retval=nf_put_att_real(ncid,lon_bid,
     &                        'valid_min',nf_float,1,-180.0)
      retval=nf_put_att_real(ncid,lon_bid,
     &                        'valid_max',nf_float,1,+180.0)
      retval=nf_put_att_text(ncid,lon_bid,'coordinates',3,'lon')

      ! Latitude Bounds
      !-------------------------
      Vdim(1)=bndsdim
      Vdim(2)=latdim
      retval=nf_def_var(ncid,'lat_bounds',nf_float,2,Vdim,lat_bid)
      retval=nf_put_att_real(ncid,lat_bid,
     &                        'valid_min',nf_float,1,-90.0)
      retval=nf_put_att_real(ncid,lat_bid,
     &                        'valid_max',nf_float,1,+90.0)
      retval=nf_put_att_text(ncid,lat_bid,'coordinates',3,'lat')

      ! Brightness Temperature
      !--------------------------
      Vdim(1)=londim
      Vdim(2)=latdim
      Vdim(3)=timedim
      retval=nf_def_var(ncid,'bt_climatology',nf_float,3,Vdim,tbsid)
      retval=nf_put_att_text(ncid,tbsid,'units',1,'K')
      retval=nf_put_att_text(ncid,tbsid,
     &       'long_name',65,
     &       'NOAA Climate Data Record of AMSU Channel 9 '//
     &       'Brightness Temperature')
      retval=nf_put_att_text(ncid,tbsid,
     &       'standard_name',34,'brightness_temperature_climatology')
      retval=nf_put_att_real(ncid,tbsid,
     &       'valid_min',nf_float,1,170.0)
      retval=nf_put_att_real(ncid,tbsid,
     &       'valid_max',nf_float,1,340.0)
      retval=nf_put_att_real(ncid,tbsid,
     &       'missing_value',nf_float,1,-999.)
        retval=nf_put_att_text(ncid,tbsid,
     &          'coordinates',12,'time lat lon')
        retval=nf_put_att_text(ncid,tbsid,
     &          'cell_methods',56,'time: mean within years '//
     &                            'time: mean over years area: mean')

      ! End DEFINE mode
      !--------------------
      retval=nf_enddef(ncid)
      
      ! write out data
      !---------------
      retval=nf_put_var_real(ncid,time_id,time)
      retval=nf_put_var_real(ncid,lon_id,lon)
      retval=nf_put_var_real(ncid,lat_id,lat)
      retval=nf_put_var_real(ncid,time_bid,climatology_bounds)
      retval=nf_put_var_real(ncid,lon_bid,lon_bounds)
      retval=nf_put_var_real(ncid,lat_bid,lat_bounds)
      retval=nf_put_var_real(ncid,tbsid,tbs)
      
      ! close netCDF file
      !------------------
      retval=nf_close(ncid)
      
!
! End Program
!---------------
      deallocate(time)
      deallocate(lat)
      deallocate(lon)
      deallocate(climatology_bounds)
      deallocate(        lat_bounds)
      deallocate(        lon_bounds)
      deallocate(tbs)
      end

      real function daysince(year0,year,Jday)
      !  daysince: Given the 'year' and Julian Day 'Jday', add 
      !            up days since Jan 01 of the given 
      !            reference 'year0'.
      !===========================================================
      integer year0,year
      real    Jday
      integer yy

      daysince=0.
      do yy=year0,(year-1)
        if((yy-2000).eq.(4*((yy-2000)/4))) then
          daysince=daysince+366.
        else
          daysince=daysince+365.
        endif
      end do
      daysince=daysince+Jday

      return
      end
