      program main
!=====================================================================
!@***h*  MW_Sounder_FCDR/gen_netCDF_monthly.f
!
! NAME
!           gen_netCDF_monthly.f
!
! LOCATION
!           $SRC_DIR/Src_fortran/
!
! DESCRIPTION: 
!              Read in gridded ASCII data for the specified months,
!              write out desired values to netCDF files. The original 
!              IDL program had to be rewritten in FORTRAN to produde
!              Version 4 netCDF files. The cumbersome usage methodology 
!              is the same as for the IDL routines for consistency.
!
! EXTERNALS:   
!              none.
!
! USAGE:       
!              The RUN parameters specifying the data to process are set 
!              directly below this header. The user needs to edit these 
!              values to process the desired values and then execute the 
!              program from within IDL.
!
! INPUTS:      
!              Gridded combined polar orbiter data values are read from an 
!              ASCII file:
!
!                $InPath/bin_YYYYMM_{chan}.dat
!
! OUTPUTS:     
!              netCDF files are written with the following directory 
!              structure and file format:
!
!                $OutPath/AMSU_{proc}_{chan}_V1.0_YYYYMM.nc
!
! AUTHOR:      
!              Xinjia Zhou / Ben Ho (?)
!
! REVISIONS:   
!              Patrick Callaghan (Formatted for CDR)
!
!
! COPYRIGHT
!              © University Corporation for Atmospheric Research (UCAR) 2011.
!              All rights reserved. Please refer to the UCAR copyright 
!              statement in the main directory
!
!@*****
!=====================================================================
      include 'netcdf.inc'

      integer,parameter::NSAT=6
      character*6   I_satnam(NSAT)
      integer       I_satnum(NSAT)
      character*300 SourceNam
      integer       SourceLen

      character*40  chan
      integer       clen
      character*130 InPath,OutPath
      integer       InPlen,OutPlen
      character*40  ProcLabT,ProcLab
      integer       ProcLenT,ProcLen
      character*9   ChanLab
      integer       YEAR_BEG,MONTH_BEG
      integer       YEAR_END,MONTH_END
   
      character*11  infbas
      character*130 infile
      integer       inflen
      character*20  time_beg,time_end
      character*7   outfbas
      character*130 outname
      integer       outnlen
      character*130 outfile
      integer       outflen

      real,   allocatable::time(:)
      real,   allocatable::lat(:)
      real,   allocatable::lon(:)
      real,   allocatable::time_bounds(:,:)
      real,   allocatable::lat_bounds(:,:)
      real,   allocatable::lon_bounds(:,:)
      real,   allocatable::sza(:,:)
      real,   allocatable::sat(:,:)
      real,   allocatable::tbs(:,:,:)
      integer,allocatable::num(:,:)
      real,   allocatable::I_lat(:)
      real,   allocatable::I_lon(:)
      real,   allocatable::I_sza(:)
      real,   allocatable::I_sat(:)
      real,   allocatable::I_tbs(:)
      integer,allocatable::I_num(:)
      integer    ,allocatable::satnum(:,:)
      character*6,allocatable::satnam(:,:)

      real    lat_loop,lon_loop
      real    lat_res,lon_res
      integer lat_num,lon_num
      integer time_num,bnds_num
      integer lat_ind,lon_ind
      integer inum,lun3,ioval,nums,mm
      integer year_loop,month_loop
      integer year_loop1,month_loop1
      integer monthstart,monthend

      character*30 timelab
      character*20 cdate
      integer values(8)
      integer retval,Vdim(3)
      integer ncid,timedim,bndsdim,londim,latdim
      integer time_id ,lat_id ,lon_id
      integer time_bid,lat_bid,lon_bid
      integer tbsid

      real    Jdays(12)     ,Jlo(12)     ,Jhi(12)
      real    Jdays_leap(12),Jlo_leap(12),Jhi_leap(12)
      data Jdays_leap /  15. ,  45. ,  75. , 105.5, 136. , 166.5,
     &                  197. , 228. , 258.5, 289. , 319.5, 350. /
      data Jdays      /  15. ,  44.5,  74. , 104.5, 135. , 165.5,
     &                  196. , 227. , 257.5, 288. , 318.5, 349. /
      data Jlo_leap /   0. ,  31. ,  60. ,  91. , 121. , 152. ,
     &                182. , 213. , 244. , 274. , 305. , 335. /
      data Jhi_leap /  30. ,  59. ,  90. , 120. , 151. , 181. ,
     &                212. , 243. , 273. , 304. , 334. , 365. /
      data Jlo      /   0. ,  31. ,  59. ,  90. , 120. , 151. ,
     &                181. , 212. , 243. , 273. , 304. , 334. /
      data Jhi      /  30. ,  58. ,  89. , 119. , 150. , 180. ,
     &                211. , 242. , 272. , 303. , 333. , 364. /

!**************************************************************************
! Set RUN parameters here:
!                 chan= String indicating the AMSU channel number and whether 
!                       the data has been converted using GPSRO data.
!                           values = {'chan9', 'chan9_converted'}
!  YEAR_BEG,MONTH_BEG = The begining year and Month to process
!  YEAR_END,MONTH_END = The ending year and Month to process
!   InPath            = The root path to the ASCII input files.
!   OutPath           = The root path for the netCDF files containing 
!                       combined/calibrated data
!
!                       Path values should contain a trailing '/'
!
!**************************************************************************
           chan='chan9_converted'
       YEAR_BEG=2001 ! 2006
      MONTH_BEG=5    ! 11
       YEAR_END=2010 ! 2006
      MONTH_END=12
         InPath='../Test-Data/ASCII_tmp/combine/'
        OutPath='../Test-Data/Output/'
!*********************************************************
! End of RUN parameters
!*********************************************************

!
! Set RUN string lengths
!------------------------
      clen=len_trim(chan)
      InPlen=len_trim(InPath)
      OutPlen=len_trim(OutPath)
!
! Set the creation date
!--------------------------
      call date_and_time(VALUES=values)
      cdate='YYYY-MM-DDTHH:MM:SSZ'
      write(cdate( 1: 4),'(i4.4)') values(1)
      write(cdate( 6: 7),'(i2.2)') values(2)
      write(cdate( 9:10),'(i2.2)') values(3)
      write(cdate(12:13),'(i2.2)') (values(5)-(values(4)/60))
      write(cdate(15:16),'(i2.2)') values(6)
      write(cdate(18:19),'(i2.2)') values(7)
!
! Set netCDF filename strings for specified {chan}
!      ProcLab = String in netCDF filename indicating the processing 
!                of data.
!                     values = {'COMBINED','CALIBRATED'}
!      ChanLab = 'CH09'
!--------------------------------------------------
      if(chan(7:15).eq.'converted') then 
        ProcLab='RO_CAL_BT_V01R00_MON_'
        ProcLabT='Calibrated_Monthly'
      else 
        ProcLab='RO_COM_BT_V01R00_MON_'
        ProcLabT='Combined_Monthly'
      endif
      ProcLen =len_trim(ProcLab)
      ProcLenT=len_trim(ProcLabT)
      if(chan(1:5).eq.'chan9') then 
        ChanLab='AMSU_CH9_'
      else 
        print *,' Unknown channel'
        STOP
      endif

! Set the Lat/Lon resolution
!------------------------------
      lat_res=2.5
      lon_res=2.5

! Get lon/lat grid dimensions, allocate space
!-----------------------------------------------
      time_num=12
      bnds_num=2
      lat_num=ifix(180./lat_res)
      lon_num=ifix(360./lon_res)
      allocate(time(time_num))
      allocate( lat(lat_num))
      allocate( lon(lon_num))
      allocate(time_bounds(bnds_num,time_num))
      allocate( lat_bounds(bnds_num,lat_num))
      allocate( lon_bounds(bnds_num,lon_num))
      allocate(sza(lon_num,lat_num))
      allocate(sat(lon_num,lat_num))
      allocate(tbs(lon_num,lat_num,time_num))
      allocate(num(lon_num,lat_num))
      allocate(I_lat(lon_num*lat_num))
      allocate(I_lon(lon_num*lat_num))
      allocate(I_sza(lon_num*lat_num))
      allocate(I_sat(lon_num*lat_num))
      allocate(I_tbs(lon_num*lat_num))
      allocate(I_num(lon_num*lat_num))
      allocate(satnum(NSAT,time_num))
      allocate(satnam(NSAT,time_num))
      tbs(:,:,:)=-999.

! Load Lat/Lon values
!----------------------
      inum=1
      lat_loop=-90. 
      do while(lat_loop.le.(90.-lat_res))  
        lat(inum)=lat_loop+(lat_res/2.)
        lat_bounds(1,inum)=lat(inum)-(lat_res/2.)
        lat_bounds(2,inum)=lat(inum)+(lat_res/2.)
        inum=inum+1
        lat_loop=lat_loop+lat_res
      end do  ! while(lat_loop.le.(90.-lat_res))

      inum=1
      lon_loop=-180.
      do while(lon_loop.le.(180.-lon_res)) 
        lon(inum)=lon_loop+(lon_res/2.)
        lon_bounds(1,inum)=lon(inum)-(lon_res/2.)
        lon_bounds(2,inum)=lon(inum)+(lon_res/2.)
        inum=inum+1
        lon_loop=lon_loop+lon_res
      end do  ! while(lon_loop.le.(180.-lon_res))

!
! Loop over the Years to process
!-------------------------------
      do year_loop=YEAR_BEG,YEAR_END

        ! Set time values for the current year
        !--------------------------------------
        if((year_loop-2000).eq.(4*((year_loop-2000)/4))) then
          do mm=1,12
            time(mm)=Jdays_leap(mm)
            time_bounds(1,mm)=Jlo_leap(mm)
            time_bounds(2,mm)=Jhi_leap(mm)
          end do
        else
          do mm=1,12
            time(mm)=Jdays(mm)
            time_bounds(1,mm)=Jlo(mm)
            time_bounds(2,mm)=Jhi(mm)
          end do
        endif

        ! Set the starting and ending month for the current year
        !---------------------------------------------------------
        monthstart=1
        monthend  =12
        if(year_loop.eq.YEAR_BEG) monthstart=MONTH_BEG
        if(year_loop.eq.YEAR_END) monthend  =MONTH_END
      
        ! Loop over months for this year
        !---------------------------------
        satnum(:,:)=0
        satnam(:,:)='      '
        do month_loop=monthstart,monthend
      
          ! Construct the input file name
          !-------------------------------
          infbas='bin_YYYYMM_'
          write(infbas(5: 8),'(i4.4)') year_loop
          write(infbas(9:10),'(i2.2)') month_loop
          infile=InPath(1:InPlen)//infbas//chan(1:clen)//'.dat'
          inflen=len_trim(infile)
          print *,infile(1:inflen)
  
          ! Construct the begining/ending time strings 
          ! for netCDF attributes
          !-----------------------------------------
          month_loop1=month_loop+1
          if(month_loop1.gt.12) then 
            month_loop1=1
            year_loop1=year_loop+1
          else 
            year_loop1=year_loop
          endif

          ! Open the input file and read in data
          !--------------------------------------
          lun3=13
  901     format(2(f8.2,1x),3(f10.3,1x),i5)
  902     format(6(a6,x,i9,4x))
          open(unit=lun3,file=infile(1:inflen),status='unknown',
     &         form='formatted',action='read')

          ! Read in header record containing observation 
          ! counts for NSAT satellites.
          !---------------------------------------------------
          read(lun3,902,iostat=ioval) 
     &                      (I_satnam(inum),I_satnum(inum),inum=1,NSAT)
          if(ioval.gt.0) then
            print *,' I/O Input error reading header: err=',ioval
            stop
          elseif(ioval.lt.0) then
            print *,' EOF reached reading header: ioval=',ioval
            stop
          endif
          satnum(:,month_loop)=I_satnum(:)
          satnam(:,month_loop)=I_satnam(:)

          ! Read in data
          !-------------------------------------
          I_tbs(:)=-999.
          nums=0
          do inum=1,(lat_num*lon_num)
            read(lun3,901,iostat=ioval) I_lat(inum),I_lon(inum),
     &                                  I_sza(inum),I_sat(inum),
     &                                  I_tbs(inum),I_num(inum)
            if(ioval.gt.0) then
              print *,' I/O Input error: err=',ioval
              stop
            elseif(ioval.lt.0) then
c             print *,' EOF reached: ioval=',ioval
              exit
            else
              nums=nums+1
            endif
          end do  ! inum=1,(lat_num*lon_num)
          close(lun3)
          if(nums.eq.0) then 
            print *,' Input file error: nums=',nums
            stop
          endif

          ! initialize values
          !-------------------------------------
          sza(:,:)=-999.
          sat(:,:)=-999.
          num(:,:)=-999.

          ! Store the data according to Lat/Lon
          !--------------------------------------
          do inum=1,nums
            lat_ind=1+ifix((I_lat(inum)+ 90.)/lat_res)
            lon_ind=1+ifix((I_lon(inum)+180.)/lon_res)
            sza(lon_ind,lat_ind)=I_sza(inum)
            sat(lon_ind,lat_ind)=I_sat(inum)
            num(lon_ind,lat_ind)=I_num(inum)
            tbs(lon_ind,lat_ind,month_loop)=I_tbs(inum)
          end do  ! inum=1,nums

        ! End loop over months
        !----------------------
        end do  ! month_loop=monthstart,monthend

        ! Generate list of source satellites
        !--------------------------------------
        do inum=1,NSAT
          I_satnum(inum)=0
          do mm=monthstart,monthend
            if(satnum(inum,mm).gt.0) then
              I_satnum(inum)=1
              I_satnam(inum)=satnam(inum,mm)
            endif
          end do
        end do


        SourceNam='UCAR COSMIC Data Analysis and Archival Center vers'
     &          //'ion 2010.2640; NOAA archive of Level 1b AMSU datas'
     &          //'et; NASA DAAC archive of Aqua Level 1b dataset;   '
     &          //'Polar Orbiters used:                              '
     &          //'                                                  '
        SourceLen=171
        do inum=1,NSAT
         if(I_satnum(inum).gt.0) then
          SourceLen=SourceLen+8
          SourceNam((SourceLen-6):SourceLen)=I_satnam(inum)
         endif
        end do
             
        ! Construct the output netCDF file name
        !---------------------------------------
        outfbas='YYYY.nc'
        write(outfbas( 1:4 ),'(i4.4)') year_loop
        outname=ChanLab//ProcLab(1:ProcLen)//outfbas
        outfile=OutPath(1:OutPlen)//outname
        outnlen=len_trim(outname)
        outflen=len_trim(outfile)
      
        ! Create netCDF file
        !-------------------------
        retval=nf_create(outfile(1:outflen),nf_clobber,ncid)
        print *,' out=',outfile(1:outflen)
        print *,' '
        if(retval.ne.nf_noerr) then
          print *,' ERROR: Opening netCDF file. retval=',retval
          stop
        endif
      
          ! Set global attributes
          !------------------------
        retval=nf_put_att_text(ncid,nf_global,
     &         'Conventions',6,'CF-1.5')
        retval=nf_put_att_text(ncid,nf_global,
     &         'title',(ProcLenT+39),ProcLabT(1:ProcLenT)//
     &         ' AMSU Channel 9 Brightness Temperatures')
        retval=nf_put_att_text(ncid,nf_global,
     &         'product_version',6,'v01r00')
        retval=nf_put_att_text(ncid,nf_global,
     &         'source',SourceLen,SourceNam(1:SourceLen))
        retval=nf_put_att_text(ncid,nf_global,
     &         'Metadata_Conventions',63,
     &         'CF-1.5, Unidata Dataset Discovery v1.0, NOAA CDR '//
     &         'v1.0, GDS v2.0')
        retval=nf_put_att_text(ncid,nf_global,
     &         'standard_name_vocabulary',43,
     &         'CF Standard Name Table (v17, 24 March 2011)')
        retval=nf_put_att_text(ncid,nf_global,
     &         'id',outnlen,outname(1:outnlen))
        retval=nf_put_att_text(ncid,nf_global,
     &         'naming_authority',13,'gov.noaa.ncdc')
        retval=nf_put_att_text(ncid,nf_global,'date_created',20,cdate)
        retval=nf_put_att_text(ncid,nf_global,
     &         'license',36,'No constraints on data access or use')
        retval=nf_put_att_text(ncid,nf_global,
     &         'summary',302,
     &         'The product is the monthly mean of adjusted AMSU ch'//
     &         'annel 9 brightness temperatures. Calculated brightn'//
     &         'ess temperatures from an AMSU forward model using t'//
     &         'emperature and moisture profiles from GPSRO observa'//
     &         'tions were used to calibrate the channel 9 measurem'//
     &         'ents from polar orbiters to a common reference.')
        retval=nf_put_att_text(ncid,nf_global,
     &         'keywords',57,
     &         'Spectral/Engineering > Microwave > Brightness '//
     &         'Temperature')
        retval=nf_put_att_text(ncid,nf_global,
     &         'keywords_vocabulary',78,
     &         'NASA Global Change Master Directory (GCMD) Earth Sc'//
     &         'ience Keywords, Version 6.0')
        retval=nf_put_att_text(ncid,nf_global,
     &         'cdm_data_type',4,'Grid')
        retval=nf_put_att_text(ncid,nf_global,
     &         'institution',11,'UCAR/COSMIC')
        retval=nf_put_att_text(ncid,nf_global,
     &         'geospatial_lat_min',6,'-88.75')
        retval=nf_put_att_text(ncid,nf_global,
     &         'geospatial_lat_max',6,'+88.75')
        retval=nf_put_att_text(ncid,nf_global,
     &         'geospatial_lon_min',7,'-178.75')
        retval=nf_put_att_text(ncid,nf_global,
     &         'geospatial_lon_max',7,'+178.75')
        retval=nf_put_att_text(ncid,nf_global,
     &         'geospatial_lat_units',13,'degrees_north')
        retval=nf_put_att_text(ncid,nf_global,
     &         'geospatial_lon_units',12,'degrees_east')
        retval=nf_put_att_text(ncid,nf_global,
     &         'geospatial_lat_resolution',3,'2.5')
        retval=nf_put_att_text(ncid,nf_global,
     &         'geospatial_lon_resolution',3,'2.5')
        time_beg='YYYY-01-01T00:00:00Z'
        time_end='YYYY-12-31T23:59:59Z'
        write(time_beg(1:4),'(i4.4)') year_loop
        write(time_end(1:4),'(i4.4)') year_loop
        retval=nf_put_att_text(ncid,nf_global,
     &         'time_coverage_start',20,time_beg)
        retval=nf_put_att_text(ncid,nf_global,
     &         'time_coverage_end',20,time_end)
        retval=nf_put_att_text(ncid,nf_global,
     &         'time_coverage_duration',3,'P1Y')
        retval=nf_put_att_text(ncid,nf_global,
     &         'time_coverage_resolution',3,'P1M')
        retval=nf_put_att_text(ncid,nf_global,
     &         'cdr_program',57,
     &         'NOAA Climate Data Record Program for satellites,'//
     &         ' FY 2011.')
        retval=nf_put_att_text(ncid,nf_global,
     &         'cdr_variable',11,'temperature')
        retval=nf_put_att_text(ncid,nf_global,
     &         'metadata_link',20,'gov.noaa.ncdc:C00807')
        retval=nf_put_att_text(ncid,nf_global,
     &        'platform',490,
     &        'EOS AQUA > Earth Observing System AQUA; METOP > Mete'//
     &        'orological Operational Satellite; NOAA-15 > National'//
     &        ' Oceanic & Atmospheric Administration-15; NOAA-16 > '//
     &        'National Oceanic & Atmospheric Administration-16; NO'//
     &        'AA-18 > National Oceanic & Atmospheric Administratio'//
     &        'n-18; NOAA-19 > National Oceanic & Atmospheric Admin'//
     &        'istration-19; CHAMP > Challenging Minisatellite Payl'//
     &        'oad, Gravity Recovery and Climate Experiment; COSMIC'//
     &        ' > Constellation Observing System for Meteorology, I'//
     &        'onosphere, and Climate')
        retval=nf_put_att_text(ncid,nf_global,
     &        'sensor',75,
     &        'AMSU-A > Advanced Micowave Sounding Unit-A; GPS > Gl'//
     &        'obal Positioning System')
        retval=nf_put_att_text(ncid,nf_global,
     &         'spatial_resolution',10,'2.5 degree')
        retval=nf_put_att_text(ncid,nf_global,
     &         'processing_level',12,'NASA Level 3')
      
        ! define dimensions
        !-------------------
        retval=nf_def_dim(ncid,'time',time_num,timedim)
        retval=nf_def_dim(ncid,'lon' , lon_num, londim)
        retval=nf_def_dim(ncid,'lat' , lat_num, latdim)
        retval=nf_def_dim(ncid,'bnds',bnds_num,bndsdim)

        ! Time Coordinates
        !-------------------------
        timelab='days since YYYY-01-01 00:00:00'
        write(timelab(12:15),'(i4.4)') year_loop
        retval=nf_def_var(ncid,'time',nf_float,1,timedim,time_id)
        retval=nf_put_att_text(ncid,time_id,'units',30,timelab)
        retval=nf_put_att_text(ncid,time_id,'long_name',4,'time')
        retval=nf_put_att_text(ncid,time_id,'standard_name',4,'time')
        retval=nf_put_att_text(ncid,time_id,'axis',1,'T')
        retval=nf_put_att_text(ncid,time_id,'bounds',11,'time_bounds')
        retval=nf_put_att_real(ncid,time_id,
     &                      'missing_value',nf_float,1,-999.)

        ! Longitude Coordinates
        !-------------------------
        retval=nf_def_var(ncid,'lon',nf_float,1,londim,lon_id)
        retval=nf_put_att_text(ncid,lon_id,'units',12,'degrees_east')
        retval=nf_put_att_text(ncid,lon_id,
     &                         'long_name',9,'longitude')
        retval=nf_put_att_text(ncid,lon_id,
     &                     'standard_name',9,'longitude')
        retval=nf_put_att_text(ncid,lon_id,'axis',1,'X')
        retval=nf_put_att_text(ncid,lon_id,'bounds',10,'lon_bounds')
        retval=nf_put_att_real(ncid,lon_id,
     &         'missing_value',nf_float,1,-999.)
      
        ! Latitude Coordinates
        !-------------------------
        retval=nf_def_var(ncid,'lat',nf_float,1,latdim,lat_id)
        retval=nf_put_att_text(ncid,lat_id,'units',13,'degrees_north')
        retval=nf_put_att_text(ncid,lat_id,
     &                         'long_name',8,'latitude')
        retval=nf_put_att_text(ncid,lat_id,
     &                     'standard_name',8,'latitude')
        retval=nf_put_att_text(ncid,lat_id,'axis',1,'Y')
        retval=nf_put_att_text(ncid,lat_id,'bounds',10,'lat_bounds')
        retval=nf_put_att_real(ncid,lat_id,
     &         'missing_value',nf_float,1,-999.)
      
        ! Time Bounds
        !-------------------------
        Vdim(1)=bndsdim
        Vdim(2)=timedim
        retval=nf_def_var(ncid,'time_bounds',nf_float,2,Vdim,time_bid)
        retval=nf_put_att_real(ncid,time_bid,
     &                           'valid_min',nf_float,1,0.)
        retval=nf_put_att_real(ncid,time_bid,
     &                           'valid_max',nf_float,1,365.)
        retval=nf_put_att_text(ncid,time_bid,'coordinates',4,'time')

        ! Longitude Bounds
        !-------------------------
        Vdim(1)=bndsdim
        Vdim(2)=londim
        retval=nf_def_var(ncid,'lon_bounds',nf_float,2,Vdim,lon_bid)
        retval=nf_put_att_real(ncid,lon_bid,
     &                          'valid_min',nf_float,1,-180.0)
        retval=nf_put_att_real(ncid,lon_bid,
     &                          'valid_max',nf_float,1,+180.0)
        retval=nf_put_att_text(ncid,lon_bid,'coordinates',3,'lon')

        ! Latitude Bounds
        !-------------------------
        Vdim(1)=bndsdim
        Vdim(2)=latdim
        retval=nf_def_var(ncid,'lat_bounds',nf_float,2,Vdim,lat_bid)
        retval=nf_put_att_real(ncid,lat_bid,
     &                          'valid_min',nf_float,1,-90.0)
        retval=nf_put_att_real(ncid,lat_bid,
     &                          'valid_max',nf_float,1,+90.0)
        retval=nf_put_att_text(ncid,lat_bid,'coordinates',3,'lat')

        ! Brightness Temperature
        !--------------------------
        Vdim(1)=londim
        Vdim(2)=latdim
        Vdim(3)=timedim
        retval=nf_def_var(ncid,'temperature',nf_float,3,Vdim,tbsid)
        retval=nf_put_att_text(ncid,tbsid,'units',1,'K')
        retval=nf_put_att_text(ncid,tbsid,
     &         'long_name',65,
     &         'NOAA Climate Data Record of AMSU Channel 9 '//
     &         'Brightness Temperature')
        retval=nf_put_att_text(ncid,tbsid,
     &         'standard_name',22,'brightness_temperature')
        retval=nf_put_att_real(ncid,tbsid,
     &         'valid_min',nf_float,1,170.0)
        retval=nf_put_att_real(ncid,tbsid,
     &         'valid_max',nf_float,1,340.0)
        retval=nf_put_att_real(ncid,tbsid,
     &         'missing_value',nf_float,1,-999.)
        retval=nf_put_att_text(ncid,tbsid,
     &          'coordinates',12,'time lat lon')
        retval=nf_put_att_text(ncid,tbsid,
     &          'cell_methods',21,'time: mean area: mean')
      
        ! End DEFINE mode
        !--------------------
        retval=nf_enddef(ncid)
      
        ! write out data
        !---------------
        retval=nf_put_var_real(ncid,time_id,time)
        retval=nf_put_var_real(ncid,lon_id,lon)
        retval=nf_put_var_real(ncid,lat_id,lat)
        retval=nf_put_var_real(ncid,time_bid,time_bounds)
        retval=nf_put_var_real(ncid,lon_bid,lon_bounds)
        retval=nf_put_var_real(ncid,lat_bid,lat_bounds)
        retval=nf_put_var_real(ncid,tbsid,tbs)
      
        ! close netCDF file
        !------------------
        retval=nf_close(ncid)
      
      end do ! year_loop=YEAR_BEG,YEAR_END 
!
! End Program
!---------------
      deallocate(time)
      deallocate(lat)
      deallocate(lon)
      deallocate(time_bounds)
      deallocate(lat_bounds)
      deallocate(lon_bounds)
      deallocate(sza)
      deallocate(sat)
      deallocate(tbs)
      deallocate(num)
      deallocate(I_lat)
      deallocate(I_lon)
      deallocate(I_sza)
      deallocate(I_sat)
      deallocate(I_tbs)
      deallocate(I_num)
      deallocate(satnum)
      deallocate(satnam)
      end
